from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import crowdstrike_process_alert_context, is_base64

crowdstrike_base64_encoded_args_tests: list[RuleTest] = [
    RuleTest(
        name="Command Line Tool Execution with Base64 Argument (Positive)",
        expected_result=True,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": '"C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe" -EncodedCommand "aGVsbG93b3JsZA==" -SomeExtraCommand "HelloWorld"',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution with Base64 Argument 2 (Positive)",
        expected_result=True,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": '"C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe" -EncodedCommand c29tZXRoaW5n -SomeExtraCommand "HelloWorld"',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution without Base64 Argument (Negative)",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": '"C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe" -WriteHost "HelloWorld123" wget test = good',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Mac - Git",
        expected_result=False,
        log={
            "ConfigBuild": "1007.4.0016304.11",
            "ConfigStateHash": "3521399940",
            "Entitlements": "15",
            "aid": "1ba46982062b43redacted",
            "aip": "1.1.1.1",
            "cid": "712bcd164963442ea43d5redacted",
            "event": {
                "CodeSigningFlags": "570503953",
                "CommandLine": "/Applications/Sourcetree.app/Contents/Resources/git_local/bin/git --no-pager --EncodedString aGVsbG93b3JsZA==",
                "ConfigBuild": "1007.4.0016304.11",
                "ConfigStateHash": "3521399940",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/Applications/Sourcetree.app/Contents/Resources/git_local/bin/git",
                "MD5HashData": "redacted",
                "MachOSubType": "1",
                "ParentBaseFileName": "Sourcetree",
                "ParentProcessId": "466750419375415990",
                "ProcessEndTime": "",
                "ProcessGroupId": "468193711305251738",
                "ProcessStartTime": "1675441378.504",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "30138",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "f154d8e18001e2fb7ae36d1eca1e833ddac057dd946fbb76ee14121a5e293538",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "466750413792797092",
                "SigningId": "git",
                "SourceProcessId": "466750419375415990",
                "SourceThreadId": "0",
                "Tags": "12094627905582, 12094627906234",
                "TargetProcessId": "468193711305251738",
                "UID": "501",
                "aid": "1ba46982062b43redacted",
                "aip": "1.1.1.1",
                "cid": "712bcd164963442ea43d5redacted",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "21fd2f8b-de5f-4ab0-b188-4bbb8c80224d",
                "name": "ProcessRollup2MacV7",
                "timestamp": "1675441380830",
            },
            "event_platform": "Mac",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "21fd2f8b-de5f-4ab0-b188-4bbb8c80224d",
            "name": "ProcessRollup2MacV7",
            "p_any_ip_addresses": ["1.1.1.1"],
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-02-03 16:23:00.83",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution without Base64 Argument (Negative) 2",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": "/c powercfg /requests",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution without Base64 Argument (Negative) 3",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": "powershell.exe  -c D:\\Atlas\\Scripts\\CopyBakToS3Job.ps1",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution without Base64 Argument (Negative) 4",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": "Powershell.exe  -NonInteractive -NoProfile -ExecutionPolicy Bypass -NoLogo -File C:\\opt\\sensu\\plugins\\check-windows-disk.ps1 90 95",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution without Base64 Argument (Negative) 5",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": 'C:\\windows\\system32\\cmd.exe  /S /D /c" ECHO F"',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution without Base64 Argument (Negative) 6",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": '"C:\\Windows\\system32\\cmd.exe" /Q /C ""C:\\Program Files (x86)\\Google\\GoogleUpdater\\128.0.6537.0\\uninstall.cmd" --dir="C:\\Program Files (x86)\\Google\\GoogleUpdater\\128.0.6537.0""',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="Command Line Tool Execution without Base64 Argument (Negative) 7",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": 'cmd /c "node -v"',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="base64 quoted argument",
        expected_result=True,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": '/usr/bin/somebinary --b64="aGVsbG8taXMtaXQtbWUteW91cmUtbG9va2luZy1mb3IK"',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="base64 prefixed argument",
        expected_result=True,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": "/usr/bin/somebinary --b64=aGVsbG8taXMtaXQtbWUteW91cmUtbG9va2luZy1mb3==",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
    RuleTest(
        name="no image name",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3645117824",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "98a4762f52a",
                "CommandLine": "/usr/bin/somebinary --b64=aGVsbG8taXMtaXQtbWUteW91cmUtbG9va2luZy1mb3==",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3645117824",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "c031e215b8b08c752bf362f6d4c5d3ad",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "1024",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682368414.719",
                "ProcessSxsFlags": "64",
                "RawProcessId": "3120",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "840e1f9dc5a29bebf01626822d7390251e9cf05bb3560ba7b68bdb8a41cf08e3",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "112532918543",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 211106232533012, 263882790666253",
                "TargetProcessId": "10413665481",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
                "name": "ProcessRollup2V19",
                "timestamp": "1682368416719",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "b0c07877-f288-49f8-8cb3-150149a557b2",
            "name": "ProcessRollup2V19",
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-24 20:33:36.719",
        },
    ),
]


@panther_managed
class CrowdstrikeBase64EncodedArgs(Rule):
    id = "Crowdstrike.Base64EncodedArgs-prototype"
    display_name = "Execution of Command Line Tool with Base64 Encoded Arguments"
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]
    tags = ["Execution", "Obfuscation"]
    default_severity = Severity.MEDIUM
    default_description = "Detects the execution of common command line tools (e.g., PowerShell, cmd.exe) with Base64 encoded arguments, which could indicate an attempt to obfuscate malicious commands."
    default_runbook = "Investigate the endpoint for signs of command line tool execution with Base64 encoded arguments. Review the executed command, decode the Base64 string, and analyze the original content."
    default_reference = (
        "https://www.crowdstrike.com/blog/blocking-fileless-script-based-attacks-using-falcon-script-control-feature/"
    )
    tests = crowdstrike_base64_encoded_args_tests
    DECODED = ""
    # List of command line tools to monitor for execution with Base64 encoded arguments
    COMMAND_LINE_TOOLS = {"powershell.exe", "cmd.exe", "cscript.exe", "wscript.exe", "rundll32.exe"}

    def rule(self, event):
        # If there is no process name available (or the CrowdStrike data model is missing) don't alert
        process_name = event.udm("process_name")
        if not process_name:
            return False
        # Filter by CS event type, Windows platform, and process name
        if not all(
            [
                event.get("fdr_event_type") == "ProcessRollup2",
                event.get("event_platform") == "Win",
                process_name.lower() in self.COMMAND_LINE_TOOLS,
            ],
        ):
            return False
        # Split arguments from process path
        command_line_args = event.udm("cmd", default="")
        command_line_args = command_line_args.replace('"', " ")
        command_line_args = command_line_args.replace("'", " ")
        command_line_args = command_line_args.replace("=", " ")
        command_line_args = command_line_args.split(" ")[1:]
        # Check if Base64 encoded arguments are present in the command line
        for arg in command_line_args:
            self.DECODED = is_base64(arg)
            if self.DECODED:
                return True
        return False

    def title(self, event):
        process_name = event.udm("process_name") if event.udm("process_name") else "Unknown"
        process_name = process_name.lower()
        command_line = event.udm("cmd")
        return f"Crowdstrike: Execution with base64 encoded args: [{process_name}] - [{command_line}]"

    def alert_context(self, event):
        context = crowdstrike_process_alert_context(event)
        context["decoded arg"] = self.DECODED
        return context



"""
Helpers for Crowdstrike Event Streams detections.
"""

from pypanther.helpers.panther_base_helpers import key_value_list_to_dict


def cs_alert_context(event):
    return key_value_list_to_dict(event.deep_get("event", "AuditKeyValues", default=[]), "Key", "ValueString")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

google_workspace_advanced_protection_program_tests: list[RuleTest] = [
    RuleTest(
        name="parameters json key set to null value",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "111111111111111111111"},
            "id": {
                "applicationName": "user_accounts",
                "customerId": "C00000000",
                "time": "2022-12-29 22:42:44.467000000",
                "uniqueQualifier": "517500000000000000",
            },
            "parameters": None,
            "ipAddress": "2600:2600:2600:2600:2600:2600:2600:2600",
            "kind": "admin#reports#activity",
            "name": "recovery_email_edit",
            "type": "recovery_info_change",
        },
    ),
    RuleTest(
        name="Allow Security Codes",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:35:29.906000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CREATE_APPLICATION_SETTING",
            "parameters": {
                "APPLICATION_EDITION": "standard",
                "APPLICATION_NAME": "Security",
                "NEW_VALUE": "ALLOWED_WITH_REMOTE_ACCESS",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "Advanced Protection Program Settings - Allow security codes",
            },
            "type": "APPLICATION_SETTINGS",
        },
    ),
    RuleTest(
        name="Enable User Enrollment",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:35:29.906000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CREATE_APPLICATION_SETTING",
            "parameters": {
                "APPLICATION_EDITION": "standard",
                "APPLICATION_NAME": "Security",
                "NEW_VALUE": "true",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "Advanced Protection Program Settings - Enable user enrollment",
            },
            "type": "APPLICATION_SETTINGS",
        },
    ),
    RuleTest(
        name="New Custom Role Created",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "123456"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 02:57:48.693000000",
                "uniqueQualifier": "-12456",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CREATE_ROLE",
            "parameters": {"ROLE_ID": "567890", "ROLE_NAME": "CustomAdminRoleName"},
            "type": "DELEGATED_ADMIN_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GoogleWorkspaceAdvancedProtectionProgram(Rule):
    default_description = "Your organization's Google Workspace Advanced Protection Program settings were modified."
    display_name = "Google Workspace Advanced Protection Program"
    default_runbook = "Confirm the changes made were authorized for your organization."
    default_reference = "https://support.google.com/a/answer/9378686?hl=en"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    id = "Google.Workspace.Advanced.Protection.Program-prototype"
    tests = google_workspace_advanced_protection_program_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        setting_name = deep_get(event, "parameters", "SETTING_NAME", default="NO_SETTING_NAME").split("-")[0].strip()
        setting_alert_flag = "Advanced Protection Program Settings"
        return event.get("name") == "CREATE_APPLICATION_SETTING" and setting_name == setting_alert_flag

    def title(self, event):
        # If no 'dedup' function is defined, the return value of this
        # method will act as deduplication string.
        setting = event.deep_get("parameters", "SETTING_NAME", default="NO_SETTING_NAME")
        setting_name = setting.split("-")[-1].strip()
        return f"Google Workspace Advanced Protection Program settings have been updated to [{setting_name}] by Google Workspace User [{event.deep_get('actor', 'email', default='<NO_EMAIL_FOUND>')}]."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

carbon_black_audit_api_key_created_retrieved_tests: list[RuleTest] = [
    RuleTest(
        name="API Key Retrieved",
        expected_result=True,
        log={
            "clientIp": "12.34.56.78",
            "description": "User bob.ross@acme.com retrieved secret for API ID JFDNIPS464 in org 12345",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Admin granted",
        expected_result=True,
        log={
            "clientIp": "12.34.56.78",
            "description": "Added API ID JFDNIPS464 with name evil-key in org 12345",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Other role granted",
        expected_result=False,
        log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Read Only",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
]


@panther_managed
class CarbonBlackAuditAPIKeyCreatedRetrieved(Rule):
    id = "CarbonBlack.Audit.API.Key.Created.Retrieved-prototype"
    log_types = [LogType.CARBON_BLACK_AUDIT]
    default_description = "Detects when a user creates a new API key or retrieves an existing key."
    display_name = "Carbon Black API Key Created or Retrieved"
    default_severity = Severity.MEDIUM
    tags = ["Persistence", "Create Account"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-F3816FB5-969F-4113-80FC-03981C65F969.html"
    tests = carbon_black_audit_api_key_created_retrieved_tests
    PATTERNS = (" retrieved secret for API ID ", "Added API ID ", "Regenerated API key for API ID ", "Updated API ID ")

    def rule(self, event):
        desc = event.get("description", "")
        return any(pattern in desc for pattern in self.PATTERNS)

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        desc = event.get("description", "<NO_DESCRIPTION_FOUND>")
        return f"{user} [{ip_addr}] {desc}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success, lookup_aws_account_name

aws_root_activity_tests: list[RuleTest] = [
    RuleTest(
        name="Root Activity - CreateServiceLinkedRole",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "Root",
                "principalId": "123456789012",
                "arn": "arn:aws:iam::123456789012:root",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {"attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"}},
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "s3.amazonaws.com",
            "eventName": "CreateServiceLinkedRole",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "bucketName": "bucket",
                "versioning": [""],
                "host": ["bucket.s3.us-west-2.amazonaws.com"],
                "VersioningConfiguration": {
                    "Status": "Enabled",
                    "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                    "MfaDelete": "Enabled",
                },
            },
            "responseElements": None,
            "additionalEventData": {
                "SignatureVersion": "SigV4",
                "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "AuthenticationMethod": "AuthHeader",
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Root Activity",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "Root",
                "principalId": "123456789012",
                "arn": "arn:aws:iam::123456789012:root",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {"attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"}},
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "s3.amazonaws.com",
            "eventName": "PutBucketVersioning",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "bucketName": "bucket",
                "versioning": [""],
                "host": ["bucket.s3.us-west-2.amazonaws.com"],
                "VersioningConfiguration": {
                    "Status": "Enabled",
                    "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                    "MfaDelete": "Enabled",
                },
            },
            "responseElements": None,
            "additionalEventData": {
                "SignatureVersion": "SigV4",
                "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "AuthenticationMethod": "AuthHeader",
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="IAMUser Activity",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "s3.amazonaws.com",
            "eventName": "GetBucketAcl",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": {"host": ["bucket.s3.us-west-2.amazonaws.com"], "bucketName": "bucket", "acl": [""]},
            "responseElements": None,
            "additionalEventData": {
                "SignatureVersion": "SigV4",
                "CipherSuite": "ECDHE-RSA-AES128-SHA",
                "AuthenticationMethod": "AuthHeader",
                "vpcEndpointId": "vpce-1",
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
            "vpcEndpointId": "vpce-1",
        },
    ),
    RuleTest(
        name="Root User Failed Activity",
        expected_result=False,
        log={
            "awsRegion": "redacted",
            "errorMessage": "Not a valid response for the provided request id: aws_ccV60redacted",
            "eventID": "redacted",
            "eventName": "ExternalIdPDirectoryLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "redacted",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.05",
            "p_alert_creation_time": "redacted",
            "p_alert_id": "redacted",
            "p_alert_update_time": "redacted",
            "p_any_aws_account_ids": ["redacted"],
            "p_any_aws_arns": [],
            "p_any_ip_addresses": ["redacted"],
            "p_event_time": "redacted",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "redacted",
            "p_row_id": "redacted",
            "p_rule_error": None,
            "p_rule_id": "AWS.Root.Activity",
            "p_rule_reports": {"CIS": ["3.3"]},
            "p_rule_tags": ["AWS", "Identity & Access Management"],
            "p_source_id": "redacted",
            "p_source_label": "CloudTrail",
            "readOnly": False,
            "recipientAccountId": "redacted",
            "requestID": "redacted",
            "responseElements": {"ExternalIdPDirectoryLogin": "Failure"},
            "sourceIPAddress": "redacted",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.105 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "",
                "accountId": "redacted",
                "arn": "",
                "principalId": "redacted",
                "type": "Root",
            },
        },
    ),
    RuleTest(
        name="Successful Root Login",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "Root",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:root",
                "accountId": "123456789012",
                "userName": "root",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSRootActivity(Rule):
    id = "AWS.Root.Activity-prototype"
    display_name = "Root Account Activity"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity & Access Management", "DemoThreatHunting", "Privilege Escalation:Valid Accounts"]
    reports = {"CIS": ["3.3"], "MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.HIGH
    default_description = "Root account activity was detected.\n"
    default_runbook = "Investigate the usage of the root account. If this root activity was not authorized, immediately change the root credentials and investigate what actions the root account took.\n"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_root-user.html"
    summary_attributes = [
        "awsRegion",
        "eventName",
        "eventSource",
        "userAgent",
        "p_any_aws_account_ids",
        "p_any_aws_arns",
        "p_any_ip_addresses",
    ]
    tests = aws_root_activity_tests
    EVENT_ALLOW_LIST = {"CreateServiceLinkedRole"}

    def rule(self, event):
        return (
            deep_get(event, "userIdentity", "type") == "Root"
            and aws_cloudtrail_success(event)
            and (deep_get(event, "userIdentity", "invokedBy") is None)
            and (event.get("eventType") != "AwsServiceEvent")
            and (event.get("eventName") not in self.EVENT_ALLOW_LIST)
        )

    def dedup(self, event):
        return (
            event.get("sourceIPAddress", "<UNKNOWN_IP>")
            + ":"
            + lookup_aws_account_name(event.get("recipientAccountId"))
            + ":"
            + str(event.get("readOnly"))
        )

    def title(self, event):
        return f"AWS root user activity [{event.get('eventName')}] in account [{lookup_aws_account_name(event.get('recipientAccountId'))}]"

    def alert_context(self, event):
        return {
            "sourceIPAddress": event.get("sourceIPAddress"),
            "userIdentityAccountId": deep_get(event, "userIdentity", "accountId"),
            "userIdentityArn": deep_get(event, "userIdentity", "arn"),
            "eventTime": event.get("eventTime"),
            "mfaUsed": deep_get(event, "additionalEventData", "MFAUsed"),
        }

    def severity(self, event):
        if event.get("readOnly"):
            return "LOW"
        return "HIGH"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gc_pserviceusageapi_keyscreate_privilege_escalation_tests: list[RuleTest] = [
    RuleTest(
        name="GCP API Key Created",
        expected_result=True,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "operations/akmf.p7-1028347275902-fe0c0688-44a7-4dca-bc06-8456068e5673",
                "last": True,
                "producer": "apikeys.googleapis.com",
            },
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@some-project.com",
                    "principalSubject": "serviceAccount:some-team@some-project.com",
                    "serviceAccountKeyName": "//iam.googleapis.com/projects/some-project/serviceAccounts/some-team@some-project.gserviceaccount.com/keys/dc5344c246064589v76ec76f66bafc92b093ed41",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "serviceusage.apiKeys.create",
                        "resource": "projectnumbers/1028347245602",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.api.apikeys.v2.ApiKeys.CreateKey",
                "request": {
                    "@type": "type.googleapis.com/google.api.apikeys.v2.CreateKeyRequest",
                    "parent": "projects/some-project/locations/global",
                },
                "requestMetadata": {
                    "callerIP": "189.163.74.177",
                    "callerSuppliedUserAgent": "(gzip),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {},
                },
                "resourceName": "projects/1028347245602",
                "response": {
                    "@type": "type.googleapis.com/google.api.apikeys.v2.Key",
                    "createTime": "1970-01-01T00:00:00Z",
                    "etag": 'W/"DSLGu9UKHwqq2ICm7YPE7g=="',
                    "name": "projects/1028347245602/locations/global/keys/bf67db25-d748-4335-ae08-7f0e65fnfy02",
                    "updateTime": "1970-01-01T00:00:00Z",
                },
                "serviceName": "apikeys.googleapis.com",
                "status": {},
            },
            "receiveTimestamp": "2024-01-25 13:28:18.961519813",
            "resource": {
                "labels": {
                    "method": "google.api.apikeys.v2.ApiKeys.CreateKey",
                    "project_id": "some-project",
                    "service": "apikeys.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-25 13:28:18.961519813",
        },
    ),
    RuleTest(
        name="GCP API Key Not Created",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "operations/akmf.p7-1028347275902-fe0c0688-44a7-4dca-bc06-8456068e5673",
                "last": True,
                "producer": "apikeys.googleapis.com",
            },
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@some-project.com",
                    "principalSubject": "serviceAccount:some-team@some-project.com",
                    "serviceAccountKeyName": "//iam.googleapis.com/projects/some-project/serviceAccounts/some-team@some-project.gserviceaccount.com/keys/dc5344c246064589v76ec76f66bafc92b093ed41",
                },
                "authorizationInfo": [
                    {
                        "granted": False,
                        "permission": "serviceusage.apiKeys.create",
                        "resource": "projectnumbers/1028347245602",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.api.apikeys.v2.ApiKeys.CreateKey",
                "request": {
                    "@type": "type.googleapis.com/google.api.apikeys.v2.CreateKeyRequest",
                    "parent": "projects/some-project/locations/global",
                },
                "requestMetadata": {
                    "callerIP": "189.163.74.177",
                    "callerSuppliedUserAgent": "(gzip),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {},
                },
                "resourceName": "projects/1028347245602",
                "response": {
                    "@type": "type.googleapis.com/google.api.apikeys.v2.Key",
                    "createTime": "1970-01-01T00:00:00Z",
                    "etag": 'W/"DSLGu9UKHwqq2ICm7YPE7g=="',
                    "name": "projects/1028347245602/locations/global/keys/bf67db25-d748-4335-ae08-7f0e65fnfy02",
                    "updateTime": "1970-01-01T00:00:00Z",
                },
                "serviceName": "apikeys.googleapis.com",
                "status": {},
            },
            "receiveTimestamp": "2024-01-25 13:28:18.961519813",
            "resource": {
                "labels": {
                    "method": "google.api.apikeys.v2.ApiKeys.CreateKey",
                    "project_id": "some-project",
                    "service": "apikeys.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-25 13:28:18.961519813",
        },
    ),
    RuleTest(
        name="Log Without authorizationInfo",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "operations/akmf.p7-1028347275902-fe0c0688-44a7-4dca-bc06-8456068e5673",
                "last": True,
                "producer": "apikeys.googleapis.com",
            },
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@some-project.com",
                    "principalSubject": "serviceAccount:some-team@some-project.com",
                    "serviceAccountKeyName": "//iam.googleapis.com/projects/some-project/serviceAccounts/some-team@some-project.gserviceaccount.com/keys/dc5344c246064589v76ec76f66bafc92b093ed41",
                },
                "methodName": "google.api.apikeys.v2.ApiKeys.CreateKey",
                "request": {
                    "@type": "type.googleapis.com/google.api.apikeys.v2.CreateKeyRequest",
                    "parent": "projects/some-project/locations/global",
                },
                "requestMetadata": {
                    "callerIP": "189.163.74.177",
                    "callerSuppliedUserAgent": "(gzip),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {},
                },
                "resourceName": "projects/1028347245602",
                "response": {
                    "@type": "type.googleapis.com/google.api.apikeys.v2.Key",
                    "createTime": "1970-01-01T00:00:00Z",
                    "etag": 'W/"DSLGu9UKHwqq2ICm7YPE7g=="',
                    "name": "projects/1028347245602/locations/global/keys/bf67db25-d748-4335-ae08-7f0e65fnfy02",
                    "updateTime": "1970-01-01T00:00:00Z",
                },
                "serviceName": "apikeys.googleapis.com",
                "status": {},
            },
            "receiveTimestamp": "2024-01-25 13:28:18.961519813",
            "resource": {},
            "severity": "NOTICE",
            "timestamp": "2024-01-25 13:28:18.961519813",
        },
    ),
]


@panther_managed
class GCPserviceusageapiKeyscreatePrivilegeEscalation(Rule):
    log_types = [LogType.GCP_AUDIT_LOG]
    default_description = "Detects serviceusage.apiKeys.create method for privilege escalation in GCP. By default, API Keys are created with no restrictions, which means they have access to the entire GCP project they were created in. We can capitalize on that fact by creating a new API key that may have more privileges than our own user."
    display_name = "GCP serviceusage.apiKeys.create Privilege Escalation"
    id = "GCP.serviceusage.apiKeys.create.Privilege.Escalation-prototype"
    default_reference = (
        "https://rhinosecuritylabs.com/cloud-security/privilege-escalation-google-cloud-platform-part-2/"
    )
    default_runbook = "Confirm this was authorized and necessary behavior. This is not a vulnerability in GCP, it is a vulnerability in how GCP environment is configured, so it is necessary to be aware of these attack vectors and to defend against them. It’s also important to remember that privilege escalation does not necessarily need to pass through the IAM service to be effective. Make sure to follow the principle of least-privilege in your environments to help mitigate these security risks."
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    default_severity = Severity.HIGH
    tests = gc_pserviceusageapi_keyscreate_privilege_escalation_tests

    def rule(self, event):
        if not deep_get(event, "protoPayload", "methodName", default="METHOD_NOT_FOUND").endswith("ApiKeys.CreateKey"):
            return False
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "serviceusage.apiKeys.create" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created new API Key in project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcpvpc_flow_logs_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Disable Flow Logs Event",
        expected_result=True,
        log={
            "insertId": "123456",
            "logName": "projects/gcp-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {"first": True, "id": "operation-abc-123", "last": True, "producer": "compute.googleapis.com"},
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-08 18:52:58.322",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-08 18:54:14.597",
            "p_source_label": "gcplogsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user1@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.update",
                        "resourceAttributes": {
                            "name": "projects/gcp-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                ],
                "methodName": "v1.compute.subnetworks.patch",
                "request": {
                    "@type": "type.googleapis.com/compute.subnetworks.patch",
                    "enableFlowLogs": False,
                    "fingerprint": "/�/��\x03��",
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {
                        "auth": {},
                        "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                        "time": "2023-03-08T18:52:58.731721Z",
                    },
                },
                "resourceName": "projects/gcp-project/regions/us-central1/subnetworks/default",
                "response": {
                    "@type": "type.googleapis.com/operation",
                    "endTime": "2023-03-08T10:52:58.700-08:00",
                    "id": "123456",
                    "insertTime": "2023-03-08T10:52:58.699-08:00",
                    "name": "operation-1678301578299-5f668096688bc-0635b6ef-4d0122bb",
                    "operationType": "compute.subnetworks.patch",
                    "progress": "100",
                    "region": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1",
                    "selfLink": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1/operations/1234",
                    "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1/operations/1234",
                    "startTime": "2023-03-08T10:52:58.700-08:00",
                    "status": "DONE",
                    "targetId": "123456",
                    "targetLink": "https://www.googleapis.com/compute/v1/projects/gcp-project/regions/us-central1/subnetworks/default",
                    "user": "user1@company.io",
                },
                "serviceName": "compute.googleapis.com",
            },
            "receiveTimestamp": "2023-03-08 18:52:58.991",
            "resource": {
                "labels": {
                    "location": "us-central1",
                    "project_id": "gcp-project",
                    "subnetwork_id": "123456",
                    "subnetwork_name": "default",
                },
                "type": "gce_subnetwork",
            },
            "severity": "NOTICE",
            "timestamp": "2023-03-08 18:52:58.322",
        },
    ),
    RuleTest(
        name="Enable Flow Logs Event",
        expected_result=False,
        log={
            "insertId": "12345",
            "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "operation-abcdefg-1234567",
                "last": True,
                "producer": "compute.googleapis.com",
            },
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-08 18:52:52.114",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-08 18:54:14.595",
            "p_row_id": "5e7586fcbb73fdeed985ebeb16bd0c",
            "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
            "p_source_label": "gcplogsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user1@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.update",
                        "resourceAttributes": {
                            "name": "projects/test-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                ],
                "methodName": "v1.compute.subnetworks.patch",
                "request": {
                    "@type": "type.googleapis.com/compute.subnetworks.patch",
                    "fingerprint": "/�/��\x03��",
                    "logConfig": {"enable": True},
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {
                        "auth": {},
                        "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                        "time": "2023-03-08T18:52:52.558899Z",
                    },
                },
                "resourceName": "projects/test-project/regions/us-central1/subnetworks/default",
                "response": {
                    "@type": "type.googleapis.com/operation",
                    "endTime": "2023-03-08T10:52:52.510-08:00",
                    "id": "9876543210",
                    "insertTime": "2023-03-08T10:52:52.509-08:00",
                    "name": "operation-abcdefg-1234567",
                    "operationType": "compute.subnetworks.patch",
                    "progress": "100",
                    "region": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1",
                    "selfLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/operation-abcdefg-1234567",
                    "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/9876543210",
                    "startTime": "2023-03-08T10:52:52.510-08:00",
                    "status": "DONE",
                    "targetId": "567854321",
                    "targetLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/subnetworks/default",
                    "user": "user1@company.io",
                },
                "serviceName": "compute.googleapis.com",
            },
            "receiveTimestamp": "2023-03-08 18:52:52.72",
            "resource": {
                "labels": {
                    "location": "us-central1",
                    "project_id": "test-project",
                    "subnetwork_id": "567854321",
                    "subnetwork_name": "default",
                },
                "type": "gce_subnetwork",
            },
            "severity": "NOTICE",
            "timestamp": "2023-03-08 18:52:52.114",
        },
    ),
]


@panther_managed
class GCPVPCFlowLogsDisabled(Rule):
    default_description = "VPC flow logs were disabled for a subnet."
    display_name = "GCP VPC Flow Logs Disabled"
    default_reference = "https://cloud.google.com/vpc/docs/using-flow-logs"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.VPC.Flow.Logs.Disabled-prototype"
    tests = gcpvpc_flow_logs_disabled_tests

    def rule(self, event):
        return all(
            [
                event.get("protoPayload"),
                deep_get(event, "protoPayload", "methodName", default="") == "v1.compute.subnetworks.patch",
                deep_get(event, "protoPayload", "request", "enableFlowLogs") is False,
            ],
        )

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        return f"GCP: [{actor}] disabled VPC Flow Logs for [{resource}]"



import json
from unittest.mock import MagicMock

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_config import config

dropbox_ownership_transfer_tests: list[RuleTest] = [
    RuleTest(
        name="Folder Ownership Transfer to External",
        expected_result=True,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
                },
            },
            "assets": [
                {
                    ".tag": "folder",
                    "display_name": "test1",
                    "path": {
                        "contextual": "/Alice Bob/test1",
                        "namespace_relative": {"is_shared_namespace": True, "ns_id": "12345"},
                    },
                },
            ],
            "context": {
                "_tag": "team_member",
                "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                "display_name": "Alice Bob",
                "email": "alice.bob@company.io",
                "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
            },
            "details": {
                ".tag": "shared_folder_transfer_ownership_details",
                "new_owner_email": "david.davidson@company.io",
                "previous_owner_email": "alice.bob@company.io",
            },
            "event_category": {"_tag": "sharing"},
            "event_type": {
                "_tag": "shared_folder_transfer_ownership",
                "description": "Transferred ownership of shared folder to another member",
            },
            "involve_non_team_member": False,
            "origin": {
                "access_method": {
                    ".tag": "end_user",
                    "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                },
                "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
            },
            "p_any_emails": ["alice.bob@company.io", "david.davidson@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_usernames": ["Alice Bob", "company"],
            "p_event_time": "2023-04-18 18:54:15",
            "p_log_type": "Dropbox.TeamEvent",
            "p_parse_time": "2023-04-18 18:56:47.418",
            "p_row_id": "0eb86fcfca9bb1cdce9defd217e1cd04",
            "p_schema_version": 0,
            "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
            "p_source_label": "dropbox1",
            "participants": [
                {
                    "user": {
                        "_tag": "team_member",
                        "account_id": "dbid:ABCD",
                        "display_name": "company",
                        "email": "david.davidson@company.io",
                        "team_member_id": "dbmid:DEFG",
                    },
                },
            ],
            "timestamp": "2023-04-18 18:54:15",
        },
    ),
    RuleTest(
        name="Other",
        expected_result=False,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
                },
            },
            "context": {
                "_tag": "team_member",
                "account_id": "dbid:AAAAAAAAAAAAAAAA",
                "display_name": "Alice Bob",
                "email": "alice.bob@company.io",
                "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
            },
            "details": {
                ".tag": "tfa_change_status_details",
                "new_value": {".tag": "disabled"},
                "previous_value": {".tag": "authenticator"},
                "used_rescue_code": True,
            },
            "event_category": {"_tag": "tfa"},
            "event_type": {
                "_tag": "tfa_change_status",
                "description": "Enabled/disabled/changed two-step verification setting",
            },
            "involve_non_team_member": False,
            "origin": {
                "access_method": {
                    ".tag": "end_user",
                    "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                },
                "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
            },
            "p_any_emails": ["alice.bob@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_usernames": ["Alice Bob"],
            "p_event_time": "2023-04-18 18:16:27",
            "p_log_type": "Dropbox.TeamEvent",
            "p_parse_time": "2023-04-18 18:18:46.808",
            "p_row_id": "0eb86fcfca9bb1cdce9defd217b8ac03",
            "p_schema_version": 0,
            "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
            "p_source_label": "dropbox1",
            "timestamp": "2023-04-18 18:16:27",
        },
    ),
    RuleTest(
        name="Folder Ownership Transfer to Internal",
        expected_result=True,
        mocks=[RuleMock(object_name="DROPBOX_TRUSTED_OWNERSHIP_DOMAINS", return_value='[\n    "example.com"\n]')],
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
                },
            },
            "assets": [
                {
                    ".tag": "folder",
                    "display_name": "test1",
                    "path": {
                        "contextual": "/Alice Bob/test1",
                        "namespace_relative": {"is_shared_namespace": True, "ns_id": "12345"},
                    },
                },
            ],
            "context": {
                "_tag": "team_member",
                "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                "display_name": "Alice Bob",
                "email": "alice.bob@company.io",
                "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
            },
            "details": {
                ".tag": "shared_folder_transfer_ownership_details",
                "new_owner_email": "david.davidson@example.com",
                "previous_owner_email": "alice.bob@company.io",
            },
            "event_category": {"_tag": "sharing"},
            "event_type": {
                "_tag": "shared_folder_transfer_ownership",
                "description": "Transferred ownership of shared folder to another member",
            },
            "involve_non_team_member": False,
            "origin": {
                "access_method": {
                    ".tag": "end_user",
                    "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                },
                "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
            },
            "p_any_emails": ["alice.bob@company.io", "david.davidson@example.com"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_usernames": ["Alice Bob", "company"],
            "p_event_time": "2023-04-18 18:54:15",
            "p_log_type": "Dropbox.TeamEvent",
            "p_parse_time": "2023-04-18 18:56:47.418",
            "p_row_id": "0eb86fcfca9bb1cdce9defd217e1cd04",
            "p_schema_version": 0,
            "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
            "p_source_label": "dropbox1",
            "participants": [
                {
                    "user": {
                        "_tag": "team_member",
                        "account_id": "dbid:ABCD",
                        "display_name": "company",
                        "email": "david.davidson@example.com",
                        "team_member_id": "dbmid:DEFG",
                    },
                },
            ],
            "timestamp": "2023-04-18 18:54:15",
        },
    ),
]


@panther_managed
class DropboxOwnershipTransfer(Rule):
    default_description = "Dropbox ownership of a document or folder has been transferred."
    display_name = "Dropbox Document/Folder Ownership Transfer"
    default_reference = "https://help.dropbox.com/share/owner"
    default_severity = Severity.HIGH
    log_types = [LogType.DROPBOX_TEAM_EVENT]
    id = "Dropbox.Ownership.Transfer-prototype"
    tests = dropbox_ownership_transfer_tests
    DROPBOX_TRUSTED_OWNERSHIP_DOMAINS = config.DROPBOX_TRUSTED_OWNERSHIP_DOMAINS

    def rule(self, event):
        return "Transferred ownership " in deep_get(event, "event_type", "description", default="")

    def title(self, event):
        actor = deep_get(event, "actor", "user", "email", default="<EMAIL_NOT_FOUND>")
        previous_owner = deep_get(event, "details", "previous_owner_email", default="<PREVIOUS_OWNER_NOT_FOUND>")
        new_owner = deep_get(event, "details", "new_owner_email", default="<NEW_OWNER_NOT_FOUND>")
        assets = event.get("assets", [{}])
        asset = [a.get("display_name", "<ASSET_NOT_FOUND>") for a in assets]
        return f"Dropbox: [{actor}] transferred ownership of [{asset}]from [{previous_owner}] to [{new_owner}]."

    def severity(self, event):
        if isinstance(self.DROPBOX_TRUSTED_OWNERSHIP_DOMAINS, MagicMock):
            self.DROPBOX_TRUSTED_OWNERSHIP_DOMAINS = set(json.loads(self.DROPBOX_TRUSTED_OWNERSHIP_DOMAINS()))  # pylint: disable=not-callable
        new_owner = deep_get(event, "details", "new_owner_email", default="<NEW_OWNER_NOT_FOUND>")
        if new_owner.split("@")[-1] not in self.DROPBOX_TRUSTED_OWNERSHIP_DOMAINS:
            return "HIGH"
        return "LOW"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_system_external_access_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk External Access Allowed By External Parties - Enabled",
        expected_result=True,
        log={
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "event": "group.request_access_settings.edit",
            "content": {"after": {"isEnabled": True}, "before": {}},
            "created": "2023-03-03T19:52:01.628Z",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk External Access Allowed By External Parties - Disabled",
        expected_result=True,
        log={
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "event": "group.request_access_settings.edit",
            "content": {"after": {}, "before": {"isEnabled": True}},
            "created": "2023-03-03T20:52:01.628Z",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Group SSO Membership sync",
        expected_result=False,
        log={
            "content": {
                "addAsOrgAdmin": [],
                "addAsOrgCollaborator": ["group.name"],
                "addAsOrgCustomRole": [],
                "addAsOrgRestrictedCollaborator": [],
                "removedOrgMemberships": [],
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


@panther_managed
class SnykSystemExternalAccess(Rule):
    id = "Snyk.System.ExternalAccess-prototype"
    display_name = "Snyk System External Access Settings Changed"
    log_types = [LogType.SNYK_GROUP_AUDIT, LogType.SNYK_ORG_AUDIT]
    tags = ["Snyk"]
    default_severity = Severity.HIGH
    default_description = "Detects when Snyk Settings that control access for external parties have been changed.\n"
    default_runbook = "This action in the Snyk Audit logs indicate that the setting for allowing external parties to request access to your Snyk installation have changed.\n"
    default_reference = "https://docs.snyk.io/snyk-admin/manage-users-and-permissions/organization-access-requests"
    summary_attributes = ["event"]
    tests = snyk_system_external_access_tests
    ACTIONS = ["group.request_access_settings.edit", "org.request_access_settings.edit"]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        current_setting = deep_get(event, "content", "after", "isEnabled", default=False)
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            action = action.split(".")[0].title()
        return f"Snyk: [{action}] External Access settings have been modified to PermitExternalUsers:[{current_setting}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        current_setting = deep_get(event, "content", "after", "isEnabled", default=False)
        a_c["current_setting"] = current_setting
        return a_c

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}"

    def severity(self, event):
        current_setting = deep_get(event, "content", "after", "isEnabled", default=False)
        if current_setting:
            return "HIGH"
        return "INFO"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_duo_helpers import deserialize_administrator_log_event_description, duo_alert_context

duo_admin_user_mfa_bypass_enabled_tests: list[RuleTest] = [
    RuleTest(
        name="Account Active",
        expected_result=False,
        log={
            "action": "user_update",
            "description": '{"status": "Active"}',
            "isotimestamp": "2021-10-05 22:45:33",
            "object": "bart.simpson@simpsons.com",
            "timestamp": "2021-10-05 22:45:33",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Account Disabled",
        expected_result=False,
        log={
            "action": "user_update",
            "description": '{"status": "Disabled"}',
            "isotimestamp": "2021-10-05 22:45:33",
            "object": "bart.simpson@simpsons.com",
            "timestamp": "2021-10-05 22:45:33",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Bypass Enabled",
        expected_result=True,
        log={
            "action": "user_update",
            "description": '{"status": "Bypass"}',
            "isotimestamp": "2021-10-05 22:45:33",
            "object": "bart.simpson@simpsons.com",
            "timestamp": "2021-10-05 22:45:33",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Phones Update",
        expected_result=False,
        log={
            "action": "user_update",
            "description": '{"phones": ""}',
            "isotimestamp": "2021-07-02 19:06:40",
            "object": "homer.simpson@simpsons.com",
            "timestamp": "2021-07-02 19:06:40",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminUserMFABypassEnabled(Rule):
    default_description = "An Administrator enabled a user to authenticate without MFA."
    display_name = "Duo Admin User MFA Bypass Enabled"
    default_reference = "https://duo.com/docs/policy#authentication-policy"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.User.MFA.Bypass.Enabled-prototype"
    tests = duo_admin_user_mfa_bypass_enabled_tests

    def rule(self, event):
        if event.get("action") == "user_update":
            description = deserialize_administrator_log_event_description(event)
            if "status" in description:
                return description.get("status") == "Bypass"
        return False

    def title(self, event):
        return f"Duo: [{event.get('username', '<username_not_found>')}] updated account [{event.get('object', '<object_not_found>')}] to not require two-factor authentication."

    def alert_context(self, event):
        return duo_alert_context(event)



from ipaddress import ip_address

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import lookup_aws_account_name

awsiam_user_recon_access_denied_tests: list[RuleTest] = [
    RuleTest(
        name="Unauthorized API Call",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "GetRole",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "40.185.186.6",
            "errorCode": "AccessDenied",
            "errorMessage": "User: arn:aws:iam::123456789012:user/tester is not authorized to perform: iam:GetRole on resource: arn:aws:iam::123456789012:role/FooBar",
            "userAgent": "aws-sdk-go/1.32.7 (go1.14.6; linux; amd64) exec-env/AWS_Lambda_go1.x",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Unauthorized API Call from Within AWS (FQDN)",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateServiceLinkedRole",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "sqs.amazonaws.com",
            "errorCode": "AccessDenied",
            "errorMessage": "User: arn:aws:iam::123456789012:user/tester is not authorized to perform: iam:Action on resource: arn:aws:iam::123456789012:resource",
            "userAgent": "sqs.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Authorized API Call",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateServiceLinkedRole",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Unauthorized API Call - From AWS console",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "GetRole",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "40.185.186.6",
            "errorCode": "AccessDenied",
            "errorMessage": "User: arn:aws:iam::123456789012:user/tester is not authorized to perform: iam:GetRole on resource: arn:aws:iam::123456789012:role/FooBar",
            "userAgent": "aws-internal/3 aws-sdk-java/1.12.124 Linux/4.9.273-0.1.ac.226.84.332.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.312-b07 java/1.8.0_312 vendor/Oracle_Corporation cfg/retry-mode/standard",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSIAMUserReconAccessDenied(Rule):
    id = "AWS.IAMUser.ReconAccessDenied-prototype"
    display_name = "Detect Reconnaissance from IAM Users"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Discovery:Cloud Service Discovery"]
    reports = {"MITRE ATT&CK": ["TA0007:T1526"]}
    default_severity = Severity.INFO
    threshold = 15
    dedup_period_minutes = 10
    default_description = "An IAM user has a high volume of access denied API calls."
    default_runbook = "Analyze the IP they came from, and other actions taken before/after."
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/troubleshoot_access-denied.html"
    summary_attributes = [
        "eventName",
        "userAgent",
        "sourceIpAddress",
        "recipientAccountId",
        "errorMessage",
        "p_any_aws_arns",
    ]
    tests = awsiam_user_recon_access_denied_tests
    # service/event patterns to monitor
    RECON_ACTIONS = {
        "dynamodb": ["List", "Describe", "Get"],
        "ec2": ["Describe", "Get"],
        "iam": ["List", "Get"],
        "s3": ["List", "Get"],
        "rds": ["Describe", "List"],
    }

    def rule(self, event):
        # Filter events
        if event.get("errorCode") != "AccessDenied":
            return False
        if deep_get(event, "userIdentity", "type") != "IAMUser":
            return False
        # Console Activity can easily result in false positives as some pages contain a mix of
        # items that a user may or may not have access to.
        if event.get("userAgent").startswith("aws-internal/3"):
            return False
        # Validate the request came from outside of AWS
        try:
            ip_address(event.get("sourceIPAddress"))
        except ValueError:
            return False
        # Pattern match this event to the recon actions
        for event_source, event_patterns in self.RECON_ACTIONS.items():
            if event.get("eventSource", "").startswith(event_source) and any(
                event.get("eventName", "").startswith(event_pattern) for event_pattern in event_patterns
            ):
                return True
        return False

    def dedup(self, event):
        return deep_get(event, "userIdentity", "arn")

    def title(self, event):
        user_type = deep_get(event, "userIdentity", "type")
        if user_type == "IAMUser":
            user = deep_get(event, "userIdentity", "userName")
        # root user
        elif user_type == "Root":
            user = user_type
        else:
            user = "<UNKNOWN_USER>"
        return f"Reconnaissance activity denied to user [{user}] in account [{lookup_aws_account_name(event.get('recipientAccountId'))}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_notion_helpers import notion_alert_context

notion_page_perms_guest_perms_changed_tests: list[RuleTest] = [
    RuleTest(
        name="Guest Role Added",
        expected_result=True,
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {
                    "entity": {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "object": "user",
                        "person": {"email": "frodo.baggins@lotr.com"},
                        "type": "person",
                    },
                    "new_permission": "full_access",
                    "old_permission": "none",
                    "page_audience": "shared_internally",
                    "target": {"page_id": "441356b5-557b-4053-8d2f-7932d2607d66", "type": "page_id"},
                },
                "id": "e18690f8-e24b-4b03-ba6f-123eb7ec0f08",
                "timestamp": "2023-08-11 23:02:53.113000000",
                "type": "page.permissions.guest_role_added",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
        },
    ),
    RuleTest(
        name="Guest Role Changed",
        expected_result=True,
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {
                    "entity": {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "object": "user",
                        "person": {"email": "frodo.baggins@lotr.com"},
                        "type": "person",
                    },
                    "new_permission": "full_access",
                    "old_permission": "read_only",
                    "page_audience": "shared_internally",
                    "target": {"page_id": "441356b5-557b-4053-8d2f-7932d2607d66", "type": "page_id"},
                },
                "id": "e18690f8-e24b-4b03-ba6f-123eb7ec0f08",
                "timestamp": "2023-08-11 23:02:53.113000000",
                "type": "page.permissions.guest_role_updated",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
        },
    ),
]


@panther_managed
class NotionPagePermsGuestPermsChanged(Rule):
    id = "Notion.PagePerms.GuestPermsChanged-prototype"
    display_name = "Notion Page Guest Permissions Changed"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Data Security", "Information Disclosure"]
    default_severity = Severity.LOW
    default_description = "The external guest permissions for a Notion page have been altered."
    default_runbook = "Potential information exposure - review the shared page and rectify if needed."
    default_reference = "https://www.notion.so/help/sharing-and-permissions"
    tests = notion_page_perms_guest_perms_changed_tests
    # These event types correspond to users adding or editing the default role on a public page
    event_types = ("page.permissions.guest_role_added", "page.permissions.guest_role_updated")

    def rule(self, event):
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") in self.event_types

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        guest = event.deep_get("event", "details", "entity", "person", "email", default="<NO_USER_FOUND>")
        page_id = event.deep_get("event", "details", "target", "page_id", default="<NO_PAGE_ID_FOUND>")
        event_type = event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
        action = {
            "page.permissions.guest_role_added": "added a guest",
            "page.permissions.guest_role_updated": "changed the guest permissions of",
        }.get(event_type, "changed the guest permissions of")
        return f"Notion User [{user}] {action} [{guest}] on page [{page_id}]."

    def alert_context(self, event):
        context = notion_alert_context(event)
        page_id = event.deep_get("event", "details", "target", "page_id", default="<NO_PAGE_ID_FOUND>")
        context["page_id"] = page_id
        details = event.deep_get("event", "details", default={})
        context["guest"] = deep_get(details, "entity", "person", "email", default="<NO_USER_FOUND>")
        context["new_permission"] = deep_get(details, "new_permission", default="<UNKNOWN PERMISSION>")
        context["old_permission"] = deep_get(details, "old_permission", default="<UNKNOWN PERMISSION>")
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

duo_admin_app_integration_secret_key_viewed_tests: list[RuleTest] = [
    RuleTest(
        name="Generic Skey View",
        expected_result=True,
        log={
            "action": "integration_skey_view",
            "isotimestamp": "2022-12-14 20:09:57",
            "object": "Example Integration Name",
            "timestamp": "2022-12-14 20:09:57",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Duo app install ",
        expected_result=False,
        log={
            "action": "application_install",
            "isotimestamp": "2022-12-14 20:09:57",
            "object": "Example Integration Name",
            "timestamp": "2022-12-14 20:09:57",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminAppIntegrationSecretKeyViewed(Rule):
    default_description = "An administrator viewed a Secret Key for an Application Integration"
    display_name = "Duo Admin App Integration Secret Key Viewed"
    default_reference = "https://duo.com/docs/adminapi"
    default_runbook = "The security of your Duo application is tied to the security of your secret key (skey). Secure it as you would any sensitive credential. Don't share it with unauthorized individuals or email it to anyone under any circumstances!"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.App.Integration.Secret.Key.Viewed-prototype"
    tests = duo_admin_app_integration_secret_key_viewed_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return event.get("action", "") == "integration_skey_view"

    def title(self, event):
        # If no 'dedup' function is defined, the return value of
        # this method will act as deduplication string.
        return f"'Duo: [{event.get('username', '<NO_USER_FOUND>')}] viewed the Secret Key for Application [{event.get('object', '<NO_OBJECT_FOUND>')}]"



from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_default import lookup_aws_account_name
from pypanther.helpers.panther_oss_helpers import geoinfo_from_ip_formatted

aws_console_root_login_tests: list[RuleTest] = [
    RuleTest(
        name="Successful Root Login",
        expected_result=True,
        mocks=[
            RuleMock(object_name="geoinfo_from_ip_formatted", return_value="111.111.111.111 in SF, California in USA"),
        ],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "Root",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:root",
                "accountId": "123456789012",
                "userName": "root",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Non-Login Event",
        expected_result=False,
        log={
            "eventVersion": "1.06",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:user/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "dynamodb.amazonaws.com",
            "eventName": "DescribeTable",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"tableName": "table"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": True,
            "resources": [{"accountId": "123456789012", "type": "AWS::DynamoDB::Table", "ARN": "arn::::table/table"}],
            "eventType": "AwsApiCall",
            "apiVersion": "2012-08-10",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSConsoleRootLogin(Rule):
    id = "AWS.Console.RootLogin-prototype"
    display_name = "Root Console Login"
    dedup_period_minutes = 15
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = [
        "AWS",
        "Identity & Access Management",
        "Authentication",
        "DemoThreatHunting",
        "Privilege Escalation:Valid Accounts",
    ]
    reports = {"CIS": ["3.6"], "MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.HIGH
    default_description = "The root account has been logged into."
    default_runbook = "Investigate the usage of the root account. If this root activity was not authorized, immediately change the root credentials and investigate what actions the root account took.\n"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_root-user.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_console_root_login_tests

    def rule(self, event):
        return (
            event.get("eventName") == "ConsoleLogin"
            and deep_get(event, "userIdentity", "type") == "Root"
            and (deep_get(event, "responseElements", "ConsoleLogin") == "Success")
        )

    def title(self, event):
        ip_address = event.get("sourceIPAddress")
        return f"AWS root login detected from [{ip_address}] ({geoinfo_from_ip_formatted(ip_address)}) in account [{lookup_aws_account_name(event.get('recipientAccountId'))}]"

    def dedup(self, event):
        # Each Root login should generate a unique alert
        return "-".join([event.get("recipientAccountId"), event.get("eventName"), event.get("eventTime")])

    def alert_context(self, event):
        return {
            "sourceIPAddress": event.get("sourceIPAddress"),
            "userIdentityAccountId": deep_get(event, "userIdentity", "accountId"),
            "userIdentityArn": deep_get(event, "userIdentity", "arn"),
            "eventTime": event.get("eventTime"),
            "mfaUsed": deep_get(event, "additionalEventData", "MFAUsed"),
        }



from panther_detection_helpers.caching import get_string_set, put_string_set

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed

git_hub_repo_initial_access_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Initial Access",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_string_set", return_value=""),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": False,
            "actor": "cat",
            "user": "",
        },
    ),
    RuleTest(
        name="GitHub - Repeated Access",
        expected_result=False,
        mocks=[RuleMock(object_name="get_string_set", return_value='"cat":"my-repo"\n')],
        log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": False,
            "actor": "cat",
            "user": "",
        },
    ),
    RuleTest(
        name="GitHub - Initial Access Public Repo",
        expected_result=False,
        mocks=[
            RuleMock(object_name="get_string_set", return_value=""),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": True,
            "actor": "cat",
            "user": "",
        },
    ),
    RuleTest(
        name="GitHub - Clone without Actor",
        expected_result=False,
        log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": False,
            "actor": "",
            "user": "",
        },
    ),
]


@panther_managed
class GitHubRepoInitialAccess(Rule):
    id = "GitHub.Repo.InitialAccess-prototype"
    display_name = "GitHub User Initial Access to Private Repo"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub"]
    default_reference = "https://docs.github.com/en/organizations/managing-user-access-to-your-organizations-repositories/managing-repository-roles/managing-an-individuals-access-to-an-organization-repository"
    default_severity = Severity.INFO
    default_description = "Detects when a user initially accesses a private organization repository."
    tests = git_hub_repo_initial_access_tests
    CODE_ACCESS_ACTIONS = ["git.clone", "git.push", "git.fetch"]

    def rule(self, event):
        # if the actor field is empty, short circuit the rule
        if not event.udm("actor_user"):
            return False
        if event.get("action") in self.CODE_ACCESS_ACTIONS and (not event.get("repository_public")):
            # Compute unique entry for this user + repo
            key = self.get_key(event)
            previous_access = get_string_set(key)
            if not previous_access:
                put_string_set(key, key)
                return True
        return False

    def title(self, event):
        return f"A user [{event.udm('actor_user')}] accessed a private repository [{event.get('repo', '<UNKNOWN_REPO>')}] for the first time."

    def get_key(self, event):
        return __name__ + ":" + str(event.udm("actor_user")) + ":" + str(event.get("repo"))



import logging

from panther_detection_helpers.caching import check_account_age

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import lookup_aws_account_name

aws_console_login_without_mfa_tests: list[RuleTest] = [
    RuleTest(
        name="No MFA Login - IAM User",
        expected_result=True,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="No MFA Login - IAM User Unknown Account",
        expected_result=True,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789013:user/tester",
                "accountId": "123456789013",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789013",
        },
    ),
    RuleTest(
        name="No MFA Login - Root User",
        expected_result=True,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "accessKeyId": "",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:root",
                "principalId": "123456789012",
                "type": "Root",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="MFA Login - SAML",
        expected_result=False,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/home",
                "MobileVersion": "No",
                "MFAUsed": "No",
                "SamlProviderArn": "arn:aws:iam::123456789012:saml-provider/Okta",
            },
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="MFA Login",
        expected_result=False,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "Yes",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="No MFA - Login Failed",
        expected_result=False,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Failure"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "Yes",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="No MFA - authenticated from session with MFA",
        expected_result=False,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "p_event_time": "2019-01-01 00:20:04.000Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "userIdentity": {
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/testrole_34c15ddd84cb4648/tester@example.com",
                "principalId": "AROAXXXXXXXXXXXXXXXXX:tester@example.com",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-09-01T21:20:03Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-east-1/testrole_34c15ddd84cb4648",
                        "principalId": "AROAXXXXXXXXXXXXXXXXX",
                        "type": "Role",
                        "userName": "testrole_34c15ddd84cb4648",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="No MFA Login - New User",
        expected_result=False,
        mocks=[RuleMock(object_name="check_account_age", return_value="True")],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="AWS SSOv2 Login",
        expected_result=False,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "additionalEventData": {"MFAUsed": "No", "MobileVersion": "No"},
            "awsRegion": "us-east-2",
            "eventID": "1111111111",
            "eventName": "ConsoleLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2022-03-16 19:17:41",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "xxx",
            "responseElements": {"ConsoleLogin": "Success"},
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.51 Safari/537.36",
            "userIdentity": {
                "accountId": "xxx",
                "arn": "arn:aws:sts::xxx:assumed-role/AWSReservedSSO_developer_admin_asdf/foo@bar.com",
                "principalId": "xxx:foo@bar.com",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-03-16T19:17:41Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "xxx",
                        "arn": "arn:aws:iam::xxx:role/aws-reserved/sso.amazonaws.com/us-east-2/AWSReservedSSO_developer_admin_asdf",
                        "principalId": "xxx",
                        "type": "Role",
                        "userName": "AWSReservedSSO_developer_admin_asdf",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="AssumeRole from MFA User Session",
        expected_result=False,
        mocks=[RuleMock(object_name="check_account_age", return_value="False")],
        log={
            "additionalEventData": {"MFAUsed": "No", "MobileVersion": "No"},
            "awsRegion": "us-east-1",
            "eventID": "sdsdsdsd",
            "eventName": "ConsoleLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2022-03-29 17:16:36.000000000",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "111111111111111",
            "responseElements": {"ConsoleLogin": "Success"},
            "sourceIPAddress": "1.1.1.1",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
            "userIdentity": {
                "accountId": "11111",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-03-29T17:16:35Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {"accountId": "2222", "type": "Role", "userName": "asdsda"},
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="No MFA - IAM Role and External IDP",
        expected_result=False,
        mocks=[RuleMock(object_name="ROLES_VIA_EXTERNAL_IDP", return_value=True)],
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/SomeRole/1641313043312360000",
                "principalId": "AROAXXXXXXXXXXXXXXXXX:1641313043312360000",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-01-04T16:17:27Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/SomeRole",
                        "principalId": "AROAXXXXXXXXXXXXXXXXX",
                        "type": "Role",
                        "userName": "SomeRole",
                    },
                },
                "type": "AssumedRole",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789013",
            "p_row_id": "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
            "p_log_type": "AWS.CloudTrail",
        },
    ),
]


@panther_managed
class AWSConsoleLoginWithoutMFA(Rule):
    id = "AWS.Console.LoginWithoutMFA-prototype"
    display_name = "Logins Without MFA"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity & Access Management", "Authentication", "Initial Access:Valid Accounts"]
    reports = {"CIS": ["3.2"], "MITRE ATT&CK": ["TA0001:T1078"]}
    default_severity = Severity.HIGH
    default_description = "A console login was made without multi-factor authentication."
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-console-login-without-mfa"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_mfa.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_console_login_without_mfa_tests
    # Set to True for environments that permit direct role assumption via external IDP
    ROLES_VIA_EXTERNAL_IDP = False
    # pylint: disable=R0911,R0912,R1260

    def rule(self, event):
        if event.get("eventName") != "ConsoleLogin":
            return False
        # Extract some nested JSON structure
        additional_event_data = event.get("additionalEventData", {})
        response_elements = event.get("responseElements", {})
        user_identity_type = deep_get(event, "userIdentity", "type", default="")
        # When there is an external IdP setup and users directly assume roles
        # the additionalData.MFAUsed attribute will be set to "no"
        #  AND the userIdentity.sessionContext.mfaAuthenticated attribute will be "false"
        #
        # This will create a lack of visibility into the condition where
        #  users are allowed to directly AssumeRole outside of the IdP and without MFA
        #
        # To date we have not identified data inside the log events that clearly
        #  delinates AssumeRole backed by an external IdP vs not backed by external IdP
        if self.ROLES_VIA_EXTERNAL_IDP and user_identity_type == "AssumedRole":
            return False
        # If using AWS SSOv2 or other SAML provider return False
        if (
            "AWSReservedSSO" in deep_get(event, "userIdentity", "arn", default=" ")
            or additional_event_data.get("SamlProviderArn") is not None
        ):
            return False
        # If Account is less than 3 days old do not alert
        # This functionality is not enabled by default, in order to start logging new user creations
        # Enable indicator_creation_rules/new_account_logging to start logging new users
        new_user_string = (
            deep_get(event, "userIdentity", "userName", default="<MISSING_USER_NAME>")
            + "-"
            + deep_get(event, "userIdentity", "principalId", default="<MISSING_ID>")
        )
        is_new_user = check_account_age(new_user_string)
        if isinstance(is_new_user, str):
            logging.debug("check_account_age is a mocked string for unit testing")
            if is_new_user == "False":
                is_new_user = False
            if is_new_user == "True":
                is_new_user = True
        if is_new_user:
            return False
        new_account_string = "new_account - " + str(event.get("recipientAccountId"))
        is_new_account = check_account_age(new_account_string)
        if isinstance(is_new_account, str):
            logging.debug("check_account_age is a mocked string for unit testing")
            if is_new_account == "False":
                is_new_account = False
            if is_new_account == "True":
                is_new_account = True
        if is_new_account:
            return False
        if response_elements.get("ConsoleLogin") == "Success":
            # This logic is inverted because at times the second condition is None.
            # It is not recommended to remove this 'double negative"
            if (
                additional_event_data.get("MFAUsed") != "Yes"
                and deep_get(event, "userIdentity", "sessionContext", "attributes", "mfaAuthenticated") != "true"
            ):
                return True
        return False

    def title(self, event):
        if deep_get(event, "userIdentity", "type") == "Root":
            user_string = "the root user"
        else:
            user = deep_get(event, "userIdentity", "userName") or deep_get(
                event,
                "userIdentity",
                "sessionContext",
                "sessionIssuer",
                "userName",
            )
            type_ = deep_get(event, "userIdentity", "sessionContext", "sessionIssuer", "type", default="user").lower()
            user_string = f"{type_} {user}"
        account_id = event.get("recipientAccountId")
        account_name = lookup_aws_account_name(account_id)
        if account_id == account_name:
            account_string = f"unnamed account ({account_id})"
        else:
            account_string = f"{account_name} account ({account_id})"
        return f"AWS login detected without MFA for [{user_string}] in [{account_string}]"

    def alert_context(self, event):
        return aws_rule_context(event)
